@@ -2,23 +2,22 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "core/fpdfapi/parser/cpdf_array.h"
-#include "core/fpdfapi/parser/cpdf_boolean.h"
-#include "core/fpdfapi/parser/cpdf_dictionary.h"
-#include "core/fpdfapi/parser/cpdf_name.h"
-#include "core/fpdfapi/parser/cpdf_null.h"
-#include "core/fpdfapi/parser/cpdf_number.h"
-#include "core/fpdfapi/parser/cpdf_reference.h"
-#include "core/fpdfapi/parser/cpdf_stream.h"
-#include "core/fpdfapi/parser/cpdf_string.h"
-
 #include <memory>
 #include <string>
 #include <utility>
 #include <vector>
 
+#include "constants/stream_dict_common.h"
+#include "core/fpdfapi/parser/cpdf_array.h"
+#include "core/fpdfapi/parser/cpdf_boolean.h"
+#include "core/fpdfapi/parser/cpdf_dictionary.h"
 #include "core/fpdfapi/parser/cpdf_indirect_object_holder.h"
-#include "core/fxcrt/fx_basic.h"
+#include "core/fpdfapi/parser/cpdf_name.h"
+#include "core/fpdfapi/parser/cpdf_null.h"
+#include "core/fpdfapi/parser/cpdf_number.h"
+#include "core/fpdfapi/parser/cpdf_reference.h"
+#include "core/fpdfapi/parser/cpdf_stream.h"
+#include "core/fpdfapi/parser/cpdf_string.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace {
@@ -82,7 +81,7 @@ class PDFObjectsTest : public testing::Test {
     // All direct objects.
     CPDF_Object* objs[] = {boolean_false_obj, boolean_true_obj, number_int_obj,
                            number_float_obj,  str_reg_obj,      str_spec_obj,
-                           name_obj,          m_ArrayObj,       m_DictObj,
+                           name_obj,          m_ArrayObj.Get(), m_DictObj.Get(),
                            stream_obj,        null_obj};
     m_DirectObjTypes = {
         CPDF_Object::BOOLEAN, CPDF_Object::BOOLEAN, CPDF_Object::NUMBER,
@@ -179,9 +178,9 @@ class PDFObjectsTest : public testing::Test {
   std::vector<std::unique_ptr<CPDF_Object>> m_DirectObjs;
   std::vector<int> m_DirectObjTypes;
   std::vector<std::unique_ptr<CPDF_Object>> m_RefObjs;
-  CPDF_Dictionary* m_DictObj;
-  CPDF_Dictionary* m_StreamDictObj;
-  CPDF_Array* m_ArrayObj;
+  UnownedPtr<CPDF_Dictionary> m_DictObj;
+  UnownedPtr<CPDF_Dictionary> m_StreamDictObj;
+  UnownedPtr<CPDF_Array> m_ArrayObj;
   std::vector<CPDF_Object*> m_IndirectObjs;
 };
 
@@ -244,23 +243,29 @@ TEST_F(PDFObjectsTest, GetInteger) {
 
 TEST_F(PDFObjectsTest, GetDict) {
   const CPDF_Dictionary* const direct_obj_results[] = {
-      nullptr, nullptr, nullptr,   nullptr,         nullptr, nullptr,
-      nullptr, nullptr, m_DictObj, m_StreamDictObj, nullptr};
+      nullptr, nullptr, nullptr, nullptr,         nullptr,
+      nullptr, nullptr, nullptr, m_DictObj.Get(), m_StreamDictObj.Get(),
+      nullptr};
   // Check for direct objects.
   for (size_t i = 0; i < m_DirectObjs.size(); ++i)
     EXPECT_EQ(direct_obj_results[i], m_DirectObjs[i]->GetDict());
 
   // Check indirect references.
-  const CPDF_Dictionary* const indirect_obj_results[] = {
-      nullptr, nullptr, nullptr, nullptr, nullptr, m_DictObj, m_StreamDictObj};
+  const CPDF_Dictionary* const indirect_obj_results[] = {nullptr,
+                                                         nullptr,
+                                                         nullptr,
+                                                         nullptr,
+                                                         nullptr,
+                                                         m_DictObj.Get(),
+                                                         m_StreamDictObj.Get()};
   for (size_t i = 0; i < m_RefObjs.size(); ++i)
     EXPECT_TRUE(Equal(indirect_obj_results[i], m_RefObjs[i]->GetDict()));
 }
 
 TEST_F(PDFObjectsTest, GetArray) {
   const CPDF_Array* const direct_obj_results[] = {
-      nullptr, nullptr,    nullptr, nullptr, nullptr, nullptr,
-      nullptr, m_ArrayObj, nullptr, nullptr, nullptr};
+      nullptr, nullptr,          nullptr, nullptr, nullptr, nullptr,
+      nullptr, m_ArrayObj.Get(), nullptr, nullptr, nullptr};
   // Check for direct objects.
   for (size_t i = 0; i < m_DirectObjs.size(); ++i)
     EXPECT_EQ(direct_obj_results[i], m_DirectObjs[i]->AsArray());
@@ -693,8 +698,9 @@ TEST(PDFArrayTest, AddInteger) {
 }
 
 TEST(PDFArrayTest, AddStringAndName) {
-  const char* vals[] = {"",        "a", "ehjhRIOYTTFdfcdnv",  "122323",
-                        "$#%^&**", " ", "This is a test.\r\n"};
+  static constexpr const char* vals[] = {
+      "",        "a", "ehjhRIOYTTFdfcdnv",  "122323",
+      "$#%^&**", " ", "This is a test.\r\n"};
   auto string_array = pdfium::MakeUnique<CPDF_Array>();
   auto name_array = pdfium::MakeUnique<CPDF_Array>();
   for (size_t i = 0; i < FX_ArraySize(vals); ++i) {
@@ -779,6 +785,81 @@ TEST(PDFArrayTest, ConvertIndirect) {
   EXPECT_EQ(42, array->GetIntegerAt(0));
 }
 
+TEST(PDFStreamTest, SetData) {
+  std::vector<uint8_t> data(100);
+  auto stream = pdfium::MakeUnique<CPDF_Stream>();
+  stream->InitStream(data.data(), data.size(),
+                     pdfium::MakeUnique<CPDF_Dictionary>());
+  EXPECT_EQ(static_cast<int>(data.size()),
+            stream->GetDict()->GetIntegerFor(pdfium::stream::kLength));
+
+  stream->GetDict()->SetNewFor<CPDF_String>(pdfium::stream::kFilter,
+                                            L"SomeFilter");
+  stream->GetDict()->SetNewFor<CPDF_String>(pdfium::stream::kDecodeParms,
+                                            L"SomeParams");
+
+  std::vector<uint8_t> new_data(data.size() * 2);
+  stream->SetData(new_data.data(), new_data.size());
+
+  // The "Length" field should be updated for new data size.
+  EXPECT_EQ(static_cast<int>(new_data.size()),
+            stream->GetDict()->GetIntegerFor(pdfium::stream::kLength));
+
+  // The "Filter" and "DecodeParms" fields should not be changed.
+  EXPECT_EQ(stream->GetDict()->GetUnicodeTextFor(pdfium::stream::kFilter),
+            L"SomeFilter");
+  EXPECT_EQ(stream->GetDict()->GetUnicodeTextFor(pdfium::stream::kDecodeParms),
+            L"SomeParams");
+}
+
+TEST(PDFStreamTest, SetDataAndRemoveFilter) {
+  std::vector<uint8_t> data(100);
+  auto stream = pdfium::MakeUnique<CPDF_Stream>();
+  stream->InitStream(data.data(), data.size(),
+                     pdfium::MakeUnique<CPDF_Dictionary>());
+  EXPECT_EQ(static_cast<int>(data.size()),
+            stream->GetDict()->GetIntegerFor(pdfium::stream::kLength));
+
+  stream->GetDict()->SetNewFor<CPDF_String>(pdfium::stream::kFilter,
+                                            L"SomeFilter");
+  stream->GetDict()->SetNewFor<CPDF_String>(pdfium::stream::kDecodeParms,
+                                            L"SomeParams");
+
+  std::vector<uint8_t> new_data(data.size() * 2);
+  stream->SetDataAndRemoveFilter(new_data.data(), new_data.size());
+  // The "Length" field should be updated for new data size.
+  EXPECT_EQ(static_cast<int>(new_data.size()),
+            stream->GetDict()->GetIntegerFor(pdfium::stream::kLength));
+
+  // The "Filter" and "DecodeParms" should be removed.
+  EXPECT_FALSE(stream->GetDict()->KeyExist(pdfium::stream::kFilter));
+  EXPECT_FALSE(stream->GetDict()->KeyExist(pdfium::stream::kDecodeParms));
+}
+
+TEST(PDFStreamTest, LengthInDictionaryOnCreate) {
+  static constexpr uint32_t kBufSize = 100;
+  // The length field should be created on stream create.
+  {
+    std::unique_ptr<uint8_t, FxFreeDeleter> data;
+    data.reset(FX_Alloc(uint8_t, kBufSize));
+    auto stream = pdfium::MakeUnique<CPDF_Stream>(
+        std::move(data), kBufSize, pdfium::MakeUnique<CPDF_Dictionary>());
+    EXPECT_EQ(static_cast<int>(kBufSize),
+              stream->GetDict()->GetIntegerFor(pdfium::stream::kLength));
+  }
+  // The length field should be corrected on stream create.
+  {
+    std::unique_ptr<uint8_t, FxFreeDeleter> data;
+    data.reset(FX_Alloc(uint8_t, kBufSize));
+    auto dict = pdfium::MakeUnique<CPDF_Dictionary>();
+    dict->SetNewFor<CPDF_Number>(pdfium::stream::kLength, 30000);
+    auto stream = pdfium::MakeUnique<CPDF_Stream>(std::move(data), kBufSize,
+                                                  std::move(dict));
+    EXPECT_EQ(static_cast<int>(kBufSize),
+              stream->GetDict()->GetIntegerFor(pdfium::stream::kLength));
+  }
+}
+
 TEST(PDFDictionaryTest, CloneDirectObject) {
   CPDF_IndirectObjectHolder objects_holder;
   auto dict = pdfium::MakeUnique<CPDF_Dictionary>();
@@ -877,3 +958,13 @@ TEST(PDFDictionaryTest, ConvertIndirect) {
   EXPECT_EQ(pObj, pNum);
   EXPECT_EQ(42, dict->GetIntegerFor("clams"));
 }
+
+TEST(PDFDictionaryTest, ExtractObjectOnRemove) {
+  auto dict = pdfium::MakeUnique<CPDF_Dictionary>();
+  CPDF_Object* pObj = dict->SetNewFor<CPDF_Number>("child", 42);
+  auto extracted_object = dict->RemoveFor("child");
+  EXPECT_EQ(pObj, extracted_object.get());
+
+  extracted_object = dict->RemoveFor("non_exists_object");
+  EXPECT_FALSE(extracted_object);
+}
