@@ -15,6 +15,7 @@
 #include "core/fxcrt/fx_coordinates.h"
 #include "core/fxcrt/fx_string.h"
 #include "core/fxcrt/unowned_ptr.h"
+#include "third_party/base/optional.h"
 
 class CPDF_Font;
 class CPDF_FormObject;
@@ -48,31 +49,26 @@ class FPDF_CHAR_INFO {
   FPDF_CHAR_INFO();
   ~FPDF_CHAR_INFO();
 
-  wchar_t m_Unicode;
-  wchar_t m_Charcode;
-  int32_t m_Flag;
-  float m_FontSize;
+  wchar_t m_Unicode = 0;
+  wchar_t m_Charcode = 0;
+  int32_t m_Flag = 0;
+  float m_FontSize = 0;
   CFX_PointF m_Origin;
   CFX_FloatRect m_CharBox;
   UnownedPtr<CPDF_TextObject> m_pTextObj;
   CFX_Matrix m_Matrix;
 };
 
-struct FPDF_SEGMENT {
-  int m_Start;
-  int m_nCount;
-};
-
 class PAGECHAR_INFO {
  public:
   PAGECHAR_INFO();
   PAGECHAR_INFO(const PAGECHAR_INFO&);
   ~PAGECHAR_INFO();
 
-  int m_Index;
-  int m_CharCode;
-  wchar_t m_Unicode;
-  int32_t m_Flag;
+  int m_Index = 0;
+  int m_CharCode = 0;
+  wchar_t m_Unicode = 0;
+  int32_t m_Flag = 0;
   CFX_PointF m_Origin;
   CFX_FloatRect m_CharBox;
   UnownedPtr<CPDF_TextObject> m_pTextObj;
@@ -93,7 +89,6 @@ class CPDF_TextPage {
   CPDF_TextPage(const CPDF_Page* pPage, FPDFText_Direction flags);
   ~CPDF_TextPage();
 
-  // IPDF_TextPage:
   void ParseTextPage();
   bool IsParsed() const { return m_bIsParsed; }
   int CharIndexFromTextIndex(int TextIndex) const;
@@ -103,16 +98,15 @@ class CPDF_TextPage {
   std::vector<CFX_FloatRect> GetRectArray(int start, int nCount) const;
   int GetIndexAtPos(const CFX_PointF& point, const CFX_SizeF& tolerance) const;
   WideString GetTextByRect(const CFX_FloatRect& rect) const;
-  WideString GetPageText(int start = 0, int nCount = -1) const;
-  int CountRects(int start, int nCount);
-  void GetRect(int rectIndex,
-               float& left,
-               float& top,
-               float& right,
-               float& bottom) const;
 
-  static bool IsRectIntersect(const CFX_FloatRect& rect1,
-                              const CFX_FloatRect& rect2);
+  // Returns string with the text from |m_TextBuf| that are covered by the input
+  // range. |start| and |count| are in terms of the |m_CharIndex|, so the range
+  // will be converted into appropriate indices.
+  WideString GetPageText(int start, int count) const;
+  WideString GetAllPageText() const { return GetPageText(0, CountChars()); }
+
+  int CountRects(int start, int nCount);
+  bool GetRect(int rectIndex, CFX_FloatRect* pRect) const;
 
  private:
   enum class TextOrientation {
@@ -140,28 +134,24 @@ class CPDF_TextPage {
                          CPDF_PageObjectList::const_iterator ObjPos);
   GenerateCharacter ProcessInsertObject(const CPDF_TextObject* pObj,
                                         const CFX_Matrix& formMatrix);
-  bool GenerateCharInfo(wchar_t unicode, PAGECHAR_INFO& info);
+  const PAGECHAR_INFO* GetPrevCharInfo() const;
+  Optional<PAGECHAR_INFO> GenerateCharInfo(wchar_t unicode);
   bool IsSameAsPreTextObject(CPDF_TextObject* pTextObj,
                              const CPDF_PageObjectList* pObjList,
                              CPDF_PageObjectList::const_iterator ObjPos);
   bool IsSameTextObject(CPDF_TextObject* pTextObj1, CPDF_TextObject* pTextObj2);
-  int GetCharWidth(uint32_t charCode, CPDF_Font* pFont) const;
   void CloseTempLine();
   FPDFText_MarkedContent PreMarkedContent(PDFTEXT_Obj pObj);
   void ProcessMarkedContent(PDFTEXT_Obj pObj);
-  void CheckMarkedContentObject(int32_t& start, int32_t& nCount) const;
+  void CheckMarkedContentObject(int32_t* pStart, int32_t* pCount) const;
   void FindPreviousTextObject();
-  void AddCharInfoByLRDirection(wchar_t wChar, PAGECHAR_INFO info);
-  void AddCharInfoByRLDirection(wchar_t wChar, PAGECHAR_INFO info);
+  void AddCharInfoByLRDirection(wchar_t wChar, const PAGECHAR_INFO& info);
+  void AddCharInfoByRLDirection(wchar_t wChar, const PAGECHAR_INFO& info);
   TextOrientation GetTextObjectWritingMode(
       const CPDF_TextObject* pTextObj) const;
   TextOrientation FindTextlineFlowOrientation() const;
   void AppendGeneratedCharacter(wchar_t unicode, const CFX_Matrix& formMatrix);
-
   void SwapTempTextBuf(int32_t iCharListStartAppend, int32_t iBufStartAppend);
-  bool IsRightToLeft(const CPDF_TextObject* pTextObj,
-                     const CPDF_Font* pFont,
-                     int nItems) const;
 
   UnownedPtr<const CPDF_Page> const m_pPage;
   std::vector<uint16_t> m_CharIndex;
@@ -172,11 +162,11 @@ class CPDF_TextPage {
   const FPDFText_Direction m_parserflag;
   UnownedPtr<CPDF_TextObject> m_pPreTextObj;
   CFX_Matrix m_perMatrix;
-  bool m_bIsParsed;
+  bool m_bIsParsed = false;
   CFX_Matrix m_DisplayMatrix;
   std::vector<CFX_FloatRect> m_SelRects;
   std::vector<PDFTEXT_Obj> m_LineObj;
-  TextOrientation m_TextlineDir;
+  TextOrientation m_TextlineDir = TextOrientation::Unknown;
   CFX_FloatRect m_CurlineRect;
 };
 
