@@ -9,10 +9,6 @@
 
 #include <memory>
 
-#include "core/fxge/cfx_fontmgr.h"
-#include "core/fxge/fx_font.h"
-
-class CCodec_ModuleMgr;
 class CFX_FontCache;
 class CFX_FontMgr;
 
@@ -21,17 +17,12 @@ class CFX_GEModule {
   static CFX_GEModule* Get();
   static void Destroy();
 
-  void Init(const char** pUserFontPaths, CCodec_ModuleMgr* pCodecModule);
+  void Init(const char** pUserFontPaths);
   CFX_FontCache* GetFontCache();
   CFX_FontMgr* GetFontMgr() { return m_pFontMgr.get(); }
-  void SetTextGamma(float gammaValue);
-  const uint8_t* GetTextGammaTable() const;
 
-  CCodec_ModuleMgr* GetCodecModule() { return m_pCodecModule; }
   void* GetPlatformData() { return m_pPlatformData; }
 
-  FXFT_Library m_FTLibrary;
-
  private:
   CFX_GEModule();
   ~CFX_GEModule();
@@ -39,10 +30,8 @@ class CFX_GEModule {
   void InitPlatform();
   void DestroyPlatform();
 
-  uint8_t m_GammaValue[256];
-  CFX_FontCache* m_pFontCache;
+  std::unique_ptr<CFX_FontCache> m_pFontCache;
   std::unique_ptr<CFX_FontMgr> m_pFontMgr;
-  CCodec_ModuleMgr* m_pCodecModule;
   void* m_pPlatformData;
   const char** m_pUserFontPaths;
 };
