@@ -45,7 +45,7 @@ struct _PopplerMovieClass
   GObjectClass parent_class;
 };
 
-G_DEFINE_TYPE (PopplerMovie, poppler_movie, G_TYPE_OBJECT);
+G_DEFINE_TYPE (PopplerMovie, poppler_movie, G_TYPE_OBJECT)
 
 static void
 poppler_movie_finalize (GObject *object)
@@ -54,7 +54,7 @@ poppler_movie_finalize (GObject *object)
 
   if (movie->filename) {
     g_free (movie->filename);
-    movie->filename = NULL;
+    movie->filename = nullptr;
   }
 
   G_OBJECT_CLASS (poppler_movie_parent_class)->finalize (object);
@@ -74,13 +74,13 @@ poppler_movie_init (PopplerMovie *movie)
 }
 
 PopplerMovie *
-_poppler_movie_new (Movie *poppler_movie)
+_poppler_movie_new (const Movie *poppler_movie)
 {
   PopplerMovie *movie;
 
-  g_assert (poppler_movie != NULL);
+  g_assert (poppler_movie != nullptr);
 
-  movie = POPPLER_MOVIE (g_object_new (POPPLER_TYPE_MOVIE, NULL));
+  movie = POPPLER_MOVIE (g_object_new (POPPLER_TYPE_MOVIE, nullptr));
 
   movie->filename = g_strdup (poppler_movie->getFileName()->getCString());
   if (poppler_movie->getShowPoster()) {
