@@ -661,7 +661,7 @@ bool MsWrdTextStyles::readPLC(MsWrdEntry &entry, int type, MWAWVec2<long> const
   MWAWInputStreamPtr &input= m_parserState->m_input;
   input->seek(entry.end()-1, librevenge::RVNG_SEEK_SET);
   auto N=static_cast<int>(input->readULong(1));
-  if (5*(N+1) > entry.length()) {
+  if (N==0 || 5*(N+1) > entry.length()) {
     MWAW_DEBUG_MSG(("MsWrdTextStyles::readPLC: the number of plc seems odd\n"));
     return false;
   }
