@@ -111,6 +111,17 @@ WPSHeader *WPSHeader::constructHeader(RVNGInputStreamPtr &input)
 				return new WPSHeader(input, input, 0, WPS_TEXT, WPS_DOSWORD);
 			}
 		}
+		if (val[0] == 0x08 && val[1] == 0xe7)
+		{
+			WPS_DEBUG_MSG(("WPSHeader::constructHeader: Multiplan spreadsheet v1 detected\n"));
+			return new WPSHeader(input, input, 1, WPS_SPREADSHEET, WPS_MULTIPLAN);
+		}
+		if (val[0] == 0x0c && (val[1] == 0xec || val[1] == 0xed))
+		{
+			int vers=int(val[1]-0xeb);
+			WPS_DEBUG_MSG(("WPSHeader::constructHeader: Multiplan spreadsheet v%d detected\n", vers));
+			return new WPSHeader(input, input, vers, WPS_SPREADSHEET, WPS_MULTIPLAN);
+		}
 		return nullptr;
 	}
 
@@ -161,6 +172,18 @@ WPSHeader *WPSHeader::constructHeader(RVNGInputStreamPtr &input)
 			return new WPSHeader(document_contents, input, 5);
 		}
 	}
+	if (input->existsSubStream("PerfectOffice_MAIN"))
+	{
+		RVNGInputStreamPtr stream(input->getSubStreamByName("PerfectOffice_MAIN"));
+		if (stream && stream->seek(0, librevenge::RVNG_SEEK_SET) == 0 && libwps::readU16(stream)==0 &&
+		        libwps::readU8(stream)==2 && libwps::readU8(stream)==0 &&
+		        libwps::readU8(stream)==7 && libwps::readU8(stream)==0x10)
+		{
+			WPS_DEBUG_MSG(("WPSHeader::constructHeader: find a Quatto Pro wb3 spreadsheet\n"));
+			return new WPSHeader(stream, input, 1003, WPS_SPREADSHEET, WPS_QUATTRO_PRO);
+		}
+	}
+	// qpw: look for stream NativeContent_MAIN and then first offset QPW9
 
 	/* check for a lotus 123 zip file containing a WK3 and FM3
 	   or a old lotus file containing WK1 and FMT
